% Figure2_3.m
% It requires the following functions:
% rk4.m
% rkGL4.m
% pcAB3.m
% fPendul
% fKepler.m
% phiKepler
% fLor.m
% fABC.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initial and final time, and  Time step
global h
t0=0;  tf=20; 
%%%%%%%%%%%%%%%  PENDULUM    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
t=[];
erEnK4=[]; erPosK4=[]; costK4=[];
erEnN4=[]; erPosN4=[]; costN4=[];
erEnKG4=[]; erPosKG4=[]; costKG4=[];
erEnPC4=[]; erPosPC4=[]; costPC4=[];
% Initial Conditions 
x0 = [0.5 ; 0];
q0=x0(1);  p0=x0(2); 
% Energy at initial conditions
H0=x0(2)^2/2+(1-cos(x0(1)));

for i=1:8
   t=[];   xk4=[];   xg4=[]; xp4=[];  qn4=[];  pn4=[];
   N=3*2^i;  h = (tf-t0)/N;
   [t xk4]=rk4(@fPendul,t0,tf,x0,N);
   [t xg4]=rkGL4(@fPendul,t0,tf,x0,N);
   [t xp4]=pcABM3(@fPendul,t0,tf,x0,N);
   [t qn4 pn4]=rkn4(@fPendulN,t0,tf,q0,p0,N);

   errorK4 = ((xk4(:,2)).^2)./2+(1-cos(xk4(:,1))) - H0;
   errorKG4 = ((xg4(:,2)).^2)./2+(1-cos(xg4(:,1))) - H0;
   errorP4 = ((xp4(:,2)).^2)./2+(1-cos(xp4(:,1))) - H0;
   errorN4 = ((pn4(:)).^2)./2+(1-cos(qn4(:))) - H0;

   erEnK4=[erEnK4; max(abs(errorK4))]; costK4=[costK4; 4*N];
   erEnKG4=[erEnKG4; max(abs(errorKG4))]; costKG4=[costKG4; 6*N];
   erEnPC4=[erEnPC4; max(abs(errorP4))]; costPC4=[costPC4; 2*N];
   erEnN4=[erEnN4; max(abs(errorN4))]; costN4=[costN4; 3*N];
end

figure(3)
 subplot(2,2,1)
  hold on
  plot(log10(costK4),log10(erEnK4(:)),'-k','LineWidth',2)
  plot(log10(costN4),log10(erEnN4(:)),'--k','LineWidth',2)
  plot(log10(costKG4),log10(erEnKG4(:)),'-ok','LineWidth',2)
%  plot(log10(costPC4),log10(erEnPC4(:)),'-b','LineWidth',2)
  axis([1.5 3 -8 -1]) 
  xlabel('{\fontsize{12} log_{10}(N. eval.) }')   
  ylabel('{\fontsize{12} log_{10}(Err. En.)}')   
  title('{\fontsize{12} Pendulum}')   
  box

%%%%%%%%%%%%  KEPLER   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
t=[];
erEnK4=[]; erPosK4=[]; costK4=[];
erEnN4=[]; erPosN4=[]; costN4=[];
erEnKG4=[]; erPosKG4=[]; costKG4=[];
erEnPC4=[]; erPosPC4=[]; costPC4=[];

% Initial Conditions 
e=0.2;   
x0 = [1-e ; 0; 0; sqrt((1+e)/(1-e))];
q0=[x0(1); x0(2)];  p0=[x0(3); x0(4)]; 
% Energy at initial conditions
H0=((x0(3))^2+(x0(4))^2)/2-1/(sqrt((x0(1))^2+(x0(2))^2));

for i=1:6
   t=[];   xk4=[];   xg4=[]; xp4=[];  qn4=[];  pn4=[];
   N=10*2^i;  h = (tf-t0)/N;
   [t xk4]=rk4(@fKepler,t0,tf,x0,N);
   [t xg4]=rkGL4(@fKepler,t0,tf,x0,N);
   [t xp4]=pcABM3(@fKepler,t0,tf,x0,N);
   [t qn4 pn4]=rkn4(@fKeplerN,t0,tf,q0,p0,N);

   errorK4 = ((xk4(:,3)).^2+(xk4(:,4)).^2)./2-1./(sqrt((xk4(:,1)).^2+(xk4(:,2)).^2)) - H0;
   errorKG4 = ((xg4(:,3)).^2+(xg4(:,4)).^2)./2-1./(sqrt((xg4(:,1)).^2+(xg4(:,2)).^2)) - H0;
   errorP4 = ((xp4(:,3)).^2+(xp4(:,4)).^2)./2-1./(sqrt((xp4(:,1)).^2+(xp4(:,2)).^2)) - H0;
   errorN4 = ((pn4(:,1)).^2+(pn4(:,2)).^2)./2-1./(sqrt((qn4(:,1)).^2+(qn4(:,2)).^2)) - H0;

   erEnK4=[erEnK4; max(abs(errorK4))]; costK4=[costK4; 4*N];
   erEnKG4=[erEnKG4; max(abs(errorKG4))]; costKG4=[costKG4; 6*N];
   erEnPC4=[erEnPC4; max(abs(errorP4))]; costPC4=[costPC4; 2*N];
   erEnN4=[erEnN4; max(abs(errorN4))]; costN4=[costN4; 3*N];
end

 subplot(2,2,2)
  hold on
  plot(log10(costK4),log10(erEnK4(:)),'-k','LineWidth',1.5)
  plot(log10(costN4),log10(erEnN4(:)),'--k','LineWidth',1.5)
  plot(log10(costKG4),log10(erEnKG4(:)),'-ok','LineWidth',1.5)
%  plot(log10(costPC4),log10(erEnPC4(:)),'-b','LineWidth',2)
  axis([2.2 3 -6 -2]) 
  xlabel('{\fontsize{12} log_{10}(N. eval.) }')   
  ylabel('{\fontsize{12} log_{10}(Err. En.)}')   
  title('{\fontsize{12} Kepler}')   
  box

%%%%%%%%%%%%%  LORENZ   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
t0=0;  tf=5; 
erEnPC4=[]; erPosPC4=[]; costPC4=[];
erEnGL4=[]; erPosGL4=[]; costGL4=[];
erEnP4=[]; erPosP4=[]; costP4=[];
% Initial Conditions 
x0 = [1 ; 2; 3];
   t=[]; x=[];
% Computing the numerical "exact" solution
  options = odeset('AbsTol',1e-10,'RelTol',1e-8);
  [t,x] = ode45('fLor',[t0,tf],x0,options);

for i=1:8
   t=[]; xk1=[];    xk2=[];    xk4=[]; 
   N=20*2^i;  h = (tf-t0)/N;
   [t xk4]=rk4(@fLor,t0,tf,x0,N);
   [t xg4]=rkGL4(@fLor,t0,tf,x0,N);
   [t xpc4]=pcABM3(@fLor,t0,tf,x0,N);

   erEnPC4=[erEnPC4; norm(x(end,:)-xpc4(end,:))]; costPC4=[costPC4; 2*N];
   erEnGL4=[erEnGL4; norm(x(end,:)-xg4(end,:))]; costGL4=[costGL4; 6*N];
   erEnP4=[erEnP4; norm(x(end,:)-xk4(end,:))]; costP4=[costP4; 4*N];
end

 subplot(2,2,3)
  hold on
%  plot(log10(costPC4),log10(erEnPC4(:)),'-b','LineWidth',2)
  plot(log10(costGL4),log10(erEnGL4(:)),'-ok','LineWidth',1.5)
  plot(log10(costP4),log10(erEnP4(:)),'-k','LineWidth',1.5)
  axis([2.5 3.3 -4 0]) 
  xlabel('{\fontsize{12} log_{10}(N. eval.) }')   
  ylabel('{\fontsize{12} log_{10}(Error)}')   
  title('{\fontsize{12} Lorenz}')   
  box

%%%%%%%%%%%%%  ABC-flow   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
t0=0;  tf=5; 
erEnPC4=[]; erPosPC4=[]; costPC4=[];
erEnGL4=[]; erPosGL4=[]; costGL4=[];
erEnP4=[]; erPosP4=[]; costP4=[];
% Initial Conditions 
x0 = [1 ; 2; 3];
   t=[]; x=[];
% Computing the numerical "exact" solution
  options = odeset('AbsTol',1e-10,'RelTol',1e-8);
  [t,x] = ode45('fABC',[t0,tf],x0,options);

for i=1:9
   t=[]; xk1=[];    xk2=[];    xk4=[]; 
   N=2*2^i;  h = (tf-t0)/N;
   [t xpc4]=pcABM3(@fABC,t0,tf,x0,N);
   [t xg4]=rkGL4(@fABC,t0,tf,x0,N);
   [t xk4]=rk4(@fABC,t0,tf,x0,N);

   erEnPC4=[erEnPC4; norm(x(end,:)-xpc4(end,:))]; costPC4=[costPC4; 2*N];
   erEnGL4=[erEnGL4; norm(x(end,:)-xg4(end,:))]; costGL4=[costGL4; 6*N];
   erEnP4=[erEnP4; norm(x(end,:)-xk4(end,:))]; costP4=[costP4; 4*N];
end

 subplot(2,2,4)
  hold on
%  plot(log10(costPC4),log10(erEnPC4(:)),'-b','LineWidth',2)
  plot(log10(costGL4),log10(erEnGL4(:)),'-ok','LineWidth',2)
  plot(log10(costP4),log10(erEnP4(:)),'-k','LineWidth',2)
  axis([1.6 2.4 -4 -1]) 
  xlabel('{\fontsize{12} log_{10}(N. eval.) }')   
  ylabel('{\fontsize{12} log_{10}(Error)}')   
  title('{\fontsize{12} ABC-flow}')   
  box

