%             rkGL4.m
% Implicit RKGL4 method
% f: vectorfield
% x0: initial conditions
% t0, tf: time span
% N: divisions of the time interval (timestep h=(tf-t0)/N)
function [t x] = rkGL4(f,t0,tf,x0,N)
 h = (tf-t0)/N; tol=1.e-10;
 t=t0:h:tf; x=x0';y=x0;
 r3=sqrt(3)/6;
 c1=1/2-r3;   a11=1/4;      a12=1/4-r3;
 c2=1/2+r3;   a21=1/4+r3;   a22=1/4; 
              b1=1/2;       b2=1/2;
 K1= f(t(1),y); K2=K1;
 for i=1:N;
   K1i=K1;    K2i=K2;
	 while 1
       K1 = f(t(i)+c1*h,y+h*(a11*K1i+a12*K2i));
       K2 = f(t(i)+c1*h,y+h*(a21*K1i+a22*K2i));
       if norm(K1-K1i)+norm(K2-K2i) < tol, break, end
       K1i=K1;    K2i=K2;
     end
  y = y + h*(b1*K1+b2*K2);
  x = [x ; y' ];
  end

